// ======================================== //
//   calendar-maker 1.0                     //
//   ファイル名から年月を取得、自動作成       //
//   作成：マージシステム株式会社             //
// ======================================== //

try {

  // ドキュメントを取得
  var doc = app.activeDocument;

  // ファイル名を取得
  var fileName = doc.name;
  var regexDate = /^date-(\d{4})(\d{2})\./; // "date-" に続いて4桁の年と2桁の月があるか

  if (regexDate.test(fileName)) {

    var match = fileName.match(regexDate);

    if (match && match.length === 3) {
      var formattedYear = match[1]; // 年
      var formattedMonth = match[2]; // 月
    } else {
        alert("エラー: 月が無効です。01から12までの数字を入力してください。");
    }

    var currentYear = formattedYear;
    var currentMonth = formattedMonth;

    // ウィンドウ
    var window = new Window("dialog", "カレンダー作成");
    var text = window.add("statictext", undefined, currentYear+"年"+currentMonth+"月のカレンダーを作成します。");
    text.alignment = "left";

    var okButton = window.add("button", undefined, "OK");

    okButton.onClick = function () {
      var selectedYear = parseInt(currentYear);
      var selectedMonth = parseInt(currentMonth);
      var monthNames = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];

      createCalendar(selectedYear, selectedMonth, monthNames);
      window.close();
    };

    // ウィンドウを表示
    window.show();

    // カレンダーを作成する関数
    function createCalendar(year, month, monthNames) {

      // カレンダーの開始日を計算（日曜始まり）
      var firstDay = new Date(year, month - 1, 2);
      var dayOfWeek = firstDay.getDay();

      var prevMonthLastDay = new Date(year, month - 1, 0).getDate(); // 先月の最終日
      var daysInMonth = new Date(year, month, 0).getDate(); // 当月の日数
      
      if (dayOfWeek === 0) {
        // 月の1日が土曜日の場合、1列目の日曜日に前月の最終日の日付を表示
        dayOfWeek = 7;
      }

      // 1月と12月の調整
      if (month === 1) {
        var prevMonth = 12;
        var prevMonthYear = year - 1;
        var nextMonth = 2;
        var nextMonthYear = year;
      } else if (month === 12) {
        var prevMonth = 11;
        var prevMonthYear = year;
        var nextMonth = 1;
        var nextMonthYear = year + 1;
      } else {
        var prevMonth = month - 1;
        var prevMonthYear = year;
        var nextMonth = month + 1;
        var nextMonthYear = year;
      }

      //先月分カレンダー
      var prevFirstDay = new Date(prevMonthYear, prevMonth - 1, 2);
      var prevDayOfWeek = prevFirstDay.getDay();
      var prevPrevMonthLastDay = new Date(prevMonthYear, prevMonth - 1, 0).getDate(); // 先月の最終日
      var prevDaysInMonth = new Date(prevMonthYear, prevMonth, 0).getDate(); // 当月の日数
      if (prevDayOfWeek === 0) {
        prevDayOfWeek = 7;
      }

      //来月分カレンダー
      var nextFirstDay = new Date(nextMonthYear, nextMonth - 1, 2);
      var nextDayOfWeek = nextFirstDay.getDay();
      var nextPrevMonthLastDay = new Date(nextMonthYear, nextMonth - 1, 0).getDate(); // 先月の最終日
      var nextDaysInMonth = new Date(nextMonthYear, nextMonth, 0).getDate(); // 当月の日数
      if (nextDayOfWeek === 0) {
        nextDayOfWeek = 7;
      }

      // レイヤーを非表示にする関数
      function toggleLayerVisibility(layerName, visibility) {
        var doc = app.activeDocument;
        var layers = doc.layers;

        for (var i = 0; i < layers.length; i++) {
          var layer = layers[i];
          if (layer.name === layerName) {
            layer.visible = visibility;
            break;
          }
        }
      }

      //テキストフレームの表示非表示を切り替える関数
      function toggleTextFrameVisibility(baseName, day, visibility) {
        var doc = app.activeDocument;
        var textFrames = doc.textFrames;
        
        for (var i = 0; i < textFrames.length; i++) {
          var textFrame = textFrames[i];
          if (textFrame.name === baseName + day) {
            textFrame.hidden = !visibility; // 非表示にする
          }
        }
      }

      // すべてのテキストフレームを取得
      var textFrames = app.activeDocument.textFrames;

      // テキストフレームの名前を指定してテキストを上書き
      for (var i = 0; i < textFrames.length; i++) {
        var textFrame = textFrames[i];

        if (textFrame.name === "nowMonthNum") { // 今月
          textFrame.contents = month.toString();
        } else if (textFrame.name === "nowMonthText") {
          var englishMonthName = monthNames[month - 1];
          textFrame.contents = englishMonthName;
        } else if (textFrame.name === "nowMonthYear") {
          textFrame.contents = year.toString();
        } else if (textFrame.name === "prevMonthYear") { // 先月の年
          textFrame.contents = prevMonthYear.toString();
        } else if (textFrame.name === "prevMonthNum") {
          textFrame.contents = prevMonth.toString();
        } else if (textFrame.name === "prevMonthText") {
          var englishMonthName = monthNames[prevMonth - 1];
          textFrame.contents = englishMonthName;
        } else if (textFrame.name === "nextMonthYear") { // 翌月の年
          textFrame.contents = nextMonthYear.toString();
        } else if (textFrame.name === "nextMonthNum") {
          textFrame.contents = nextMonth.toString();
        } else if (textFrame.name === "nextMonthText") {
          var englishMonthName = monthNames[nextMonth - 1];
          textFrame.contents = englishMonthName;
        } else if (textFrame.name.indexOf("nowDate-") === 0) { // ★今月分のカレンダー日付
          var dateIndex = parseInt(textFrame.name.replace("nowDate-", ""));
          if (dateIndex >= dayOfWeek && dateIndex < dayOfWeek + daysInMonth) {
            // 当月の日付
            var day = dateIndex - dayOfWeek + 1;
            textFrame.contents = day.toString();
            // テキストの色
            if (dateIndex % 7 === 1) {
              textFrame.textRange.fillColor = setCMYK(27, 84, 68, 0); //日曜日の色（赤）
              if(dateIndex === 36){
                toggleLayerVisibility("nowlastLine-1", true);
                toggleLayerVisibility("nowDate-36", true);
              }
            } else if (dateIndex % 7 === 0) {
              textFrame.textRange.fillColor = setCMYK(67, 28, 14, 0); //土曜日の色（青）
            } else {
              textFrame.textRange.fillColor = setCMYK(73, 66, 63, 20); //平日の色（黒）
              if(dateIndex === 37){
                toggleLayerVisibility("nowlastLine-2", true);
                toggleLayerVisibility("nowDate-37", true);
              }
            }
          } else if (dateIndex < dayOfWeek) {
            // 先月の日付
            var prevMonthDay = prevMonthLastDay - (dayOfWeek - dateIndex - 1);
            textFrame.contents = prevMonthDay.toString();
            // テキストの色
            if (dateIndex === 1) {
              textFrame.textRange.fillColor = setCMYK(19, 36, 25, 0); //先月日曜日（薄い赤）
            } else {
              textFrame.textRange.fillColor = setCMYK(0, 0, 0, 40); //先月平日（薄い黒）
            }
          } else {
            // 翌月の日付
            var nextMonthDay = dateIndex - dayOfWeek - daysInMonth + 1;
            textFrame.contents = nextMonthDay.toString();
            // テキストの色
            if (dateIndex === 29 || dateIndex === 36) {
              textFrame.textRange.fillColor = setCMYK(19, 36, 25, 0); //翌月日曜日（薄い赤）
              if(dateIndex === 36){
                toggleLayerVisibility("nowlastLine-1", false);
                toggleLayerVisibility("nowDate-36", false);
              }
            } else if (dateIndex === 35) {
              textFrame.textRange.fillColor = setCMYK(42, 21, 18, 0); //翌月土曜日（薄い青）
            } else {
              textFrame.textRange.fillColor = setCMYK(0, 0, 0, 40); //翌月平日（薄い黒）
              if(dateIndex === 37){
                toggleLayerVisibility("nowlastLine-2", false);
                toggleLayerVisibility("nowDate-37", false);
              }
            }
          }
        } else if (textFrame.name.indexOf("prevMonthDate-") === 0) { // ★先月分の小さいカレンダー日付
          var dateIndexPrev = parseInt(textFrame.name.replace("prevMonthDate-", ""));
          if (dateIndexPrev >= prevDayOfWeek && dateIndexPrev < prevDayOfWeek + prevDaysInMonth) {
            // 当月の日付
            var day = dateIndexPrev - prevDayOfWeek + 1;
            textFrame.contents = day.toString();
            // レイヤーを表示する
            toggleTextFrameVisibility("prevMonthDate-", dateIndexPrev, true);
          } else if (dateIndexPrev < prevDayOfWeek) {
            // 先月の日付
            var prevMonthDay = prevPrevMonthLastDay - (prevDayOfWeek - dateIndexPrev - 1);
            textFrame.contents = prevMonthDay.toString();
            // レイヤーを非表示にする
            toggleTextFrameVisibility("prevMonthDate-", dateIndexPrev, false);
          } else {
            // 翌月の日付
            var nextMonthDay = dateIndexPrev - prevDayOfWeek - prevDaysInMonth + 1;
            textFrame.contents = nextMonthDay.toString();
            // レイヤーを非表示にする
            toggleTextFrameVisibility("prevMonthDate-", dateIndexPrev, false);
          }
        } else if (textFrame.name.indexOf("nextMonthDate-") === 0) { // ★来月分の小さいカレンダー日付
          var dateIndexNext = parseInt(textFrame.name.replace("nextMonthDate-", ""));
          if (dateIndexNext >= nextDayOfWeek && dateIndexNext < nextDayOfWeek + nextDaysInMonth) {
            // 当月の日付
            var day = dateIndexNext - nextDayOfWeek + 1;
            textFrame.contents = day.toString();
            // レイヤーを表示する
            toggleTextFrameVisibility("nextMonthDate-", dateIndexNext, true);
          } else if (dateIndexNext < nextDayOfWeek) {
            // 先月の日付
            var prevMonthDay = nextPrevMonthLastDay - (nextDayOfWeek - dateIndexNext - 1);
            textFrame.contents = prevMonthDay.toString();
            // レイヤーを非表示にする
            toggleTextFrameVisibility("nextMonthDate-", dateIndexNext, false);
          } else {
            // 翌月の日付
            var nextMonthDay = dateIndexNext - nextDayOfWeek - nextDaysInMonth + 1;
            textFrame.contents = nextMonthDay.toString();
            // レイヤーを非表示にする
            toggleTextFrameVisibility("nextMonthDate-", dateIndexNext, false);
          }
        }
      }
    }

    function setCMYK(c, m, y, k) {
      var CMYK = new CMYKColor();
      CMYK.cyan = c;
      CMYK.magenta = m;
      CMYK.yellow = y;
      CMYK.black = k;
      return CMYK;
    }

  }else{
    // 形式が正しくない場合アラートを出す
    alert("ファイル名の形式が正しくありません。\n例のように修正してください。\n例：date-202401");
  }

} catch (e) {
  alert("ファイルが開かれていません。");
}